from app import db
from unidecode import unidecode
import uuid

class Category(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(50), nullable=False, unique=False)
    slug = db.Column(db.String(150), unique=True, nullable=False)
    videos = db.relationship('Video', backref='category_videos', lazy=True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if not self.slug:
            self.slug = self.generate_slug

    @property
    def generate_slug(self):
        unique_hash = uuid.uuid4().hex
        slug = unidecode(self.name.lower()).replace(' ', '-')
        return f'{slug}-{unique_hash}'

class Video(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    title = db.Column(db.String(100), unique=False, nullable=False)
    description = db.Column(db.Text, nullable=True)
    filename = db.Column(db.String(200), nullable=False)
    preview_filename = db.Column(db.String(255), nullable=False)
    category_id = db.Column(db.Integer, db.ForeignKey('category.id'), nullable=False)
    slug = db.Column(db.String(150), unique=True, nullable=False)
    category = db.relationship('Category', back_populates='videos', overlaps='category_videos')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if not self.slug:
            self.slug = self.generate_slug

    @property
    def generate_slug(self):
        unique_hash = uuid.uuid4().hex
        slug = unidecode(self.title.lower()).replace(' ', '-')
        return f'{slug}-{unique_hash}'
